@@ -12,9 +12,9 @@ from exo_k.util.filenames import EndOfFile
 from .util.interp import linear_interpolation, interp_ind_weights, unit_convert
 from .util.cst import KBOLTZ
 from .settings import Settings
+from .util.spectral_object import Spectral_object
 
-
-class Cia_table(object):
+class Cia_table(Spectral_object):
     """A class to handle CIA opacity data tables.
     """
 
@@ -184,7 +184,7 @@ class Cia_table(object):
                 Name of the file to be written.
         """
         if not filename.lower().endswith(('.hdf5', '.h5')):
-            filename=filename+'.hdf5'
+            filename=filename+'.h5'
         f = h5py.File(filename, 'w')
         f.create_dataset("bin_centers", data=self.wns,compression="gzip")
         f.create_dataset("t", data=self.tgrid,compression="gzip")
@@ -194,7 +194,7 @@ class Cia_table(object):
         f.close()    
 
     def sample(self, wngrid, remove_zeros=False, use_grid_filter=False, **kwargs):
-        """Method to re sample a cia table to a new grid of wavenumbers (in place)
+        """Method to re sample a cia table to a new grid of wavenumbers (inplace).
 
         Parameters
         ----------
@@ -252,6 +252,11 @@ class Cia_table(object):
                 If an array is given, interpolates only within this array.
             log_interp: bool, optional
                 Whether the interpolation is linear in kdata or in log(kdata).
+
+        Returns
+        -------
+            array of shape (logp_array.size, self.Nw)
+                The interpolated kdata.
         """
         if hasattr(t_array, "__len__"):
             t_array=np.array(t_array)
@@ -337,12 +342,11 @@ class Cia_table(object):
             ax.plot(self.wns,toplot,**kwarg)
             ax.set_xlabel('Wavenumber (cm$^{-1}$)')
         ax.set_ylabel('Abs. coeff')
-        ax.grid(True)
         if xscale is not None: ax.set_xscale(xscale)
         if yscale is not None: ax.set_yscale(yscale)
 
     def convert_abs_coeff_unit(self,abs_coeff_unit='unspecified',old_abs_coeff_unit='unspecified'):
-        """Converts abs_coeff to a new unit (in place)
+        """Converts abs_coeff to a new unit (inplace).
 
         Parameters
         ----------
@@ -384,15 +388,9 @@ class Cia_table(object):
         self.abs_coeff_unit='m^5'
         return
 
-    @property
-    def wls(self):
-        """Returns the wavelength array for the bin centers
-        """
-        return 10000./self.wns
-
     def remove_zeros(self, deltalog_min_value=0.):
         """Finds zeros in the abs_coeff and set them to (10.^-deltalog_min_value)
-        times the minimum positive value in the table.
+        times the minimum positive value in the table (inplace).
         This is to be able to work in logspace. 
         """
         mask = np.zeros(self.abs_coeff.shape,dtype=bool)
