@@ -8,8 +8,9 @@ import numpy as np
 
 from .cia_table import Cia_table
 from .settings import NoFileFoundError
+from .util.spectral_object import Spectral_object
 
-class CIAdatabase(object):
+class CIAdatabase(Spectral_object):
     """Class to group :class:`~exo_k.cia_table.Cia_table` objects
     and combine them in radiative transfer
     """
@@ -69,7 +70,7 @@ class CIAdatabase(object):
 
     
     def add_cia_tables(self, *cia_tables):
-        """Adds new :class:`~exo_k.cia_table.Cia_table` objects to a CIA database.
+        """Adds new :class:`~exo_k.cia_table.Cia_table` objects to a CIA database (inplace).
 
         Parameters
         ----------
@@ -128,7 +129,7 @@ class CIAdatabase(object):
 
     def sample(self, wngrid, remove_zeros=False, use_grid_filter=False):
         """Samples all the cia_table in the database on the same wavenumber grid
-        to be able to use them in radiative transfer modules.
+        to be able to use them in radiative transfer modules (inplace).
 
         Parameters
         ----------
@@ -147,7 +148,7 @@ class CIAdatabase(object):
         self.Nw=self.wns.size
 
     def convert_to_mks(self):
-        """Converts units of all Cia_tables to MKS.
+        """Converts units of all Cia_tables to MKS (inplace).
         """
         first=True
         for mol1 in self.cia_tables.values():
