@@ -2,23 +2,32 @@
 """
 @author: jeremy leconte
 """
+from datetime import date
 import numpy as np
+import h5py
+import pkg_resources  # part of setuptools
+import astropy.units as u
 from .util.interp import rm_molec,unit_convert,interp_ind_weights,bilinear_interpolation
 from .settings import Settings
+from .util.spectral_object import Spectral_object
+from .util.molar_mass import Molar_mass
+from .util.cst import ktable_long_name_attributes
 
-class Data_table(object):
+class Data_table(Spectral_object):
     """An abstract class that will serve as a basis for Ktable and Xtable.
     This class includes all the interpolation and remapping methods.
     """
 
     __array_priority__=1000
     # this allows __rmul__ to take precedence over numpy array broadcasting in multiplications
-    # See https://stackoverflow.com/questions/40694380/forcing-multiplication-to-use-rmul-instead-of-numpy-array-mul-or-byp/44634634#44634634
+    # See https://stackoverflow.com/questions/40694380/
+    # forcing-multiplication-to-use-rmul-instead-of-numpy-array-mul-or-byp/44634634#44634634
 
     def __init__(self):
         """Initializes all attributes to `None`"""
         self.filename=None
         self.mol=None
+        self.isotopolog_id=0
         self.pgrid=None
         self.logpgrid=None
         self.tgrid=None
@@ -33,6 +42,10 @@ class Data_table(object):
                        # This will allow us to differentiate xsec from corrk
                        # when needed in the interpolation routines.
                        # Especially to reshape the output.
+        self.DOI='unknown'
+        self.sampling_method='unknown'
+        version = pkg_resources.require("exo_k")[0].version
+        self.Date_ID='exo_k-v'+version+'-'+date.today().strftime("%d/%m/%Y")
         self.Nx=None   # If we are dealing with a Qtable with a variable gas, Nx is the size of the
                        # grid along the dimension of the Volume mixing ratio of the variable gas.
                        # A None value means that we have either a regular Ktable or a Xtable.
@@ -87,7 +100,7 @@ class Data_table(object):
 
     def remove_zeros(self,deltalog_min_value=10.):
         """Finds zeros in the kdata and set them to (10.**-deltalog_min_value)
-        times the minimum positive value in the table.
+        times the minimum positive value in the table (inplace).
 
         This is to be able to work in logspace. 
 
@@ -101,7 +114,7 @@ class Data_table(object):
         self.kdata[~mask]=minvalue/(10.**deltalog_min_value)
 
     def convert_p_unit(self, p_unit='unspecified', file_p_unit='unspecified'):
-        """Converts pressure to a new unit (in place)
+        """Converts pressure to a new unit (inplace).
 
         Parameters
         ----------
@@ -122,7 +135,7 @@ class Data_table(object):
         self.logpgrid=np.log10(self.pgrid)
 
     def convert_kdata_unit(self, kdata_unit='unspecified', file_kdata_unit='unspecified'):
-        """Converts kdata to a new unit (in place)
+        """Converts kdata to a new unit (inplace).
 
         Parameters
         ----------
@@ -151,7 +164,7 @@ class Data_table(object):
         self.kdata=self.kdata*conversion_factor
     
     def convert_to_mks(self):
-        """Converts units to MKS
+        """Converts units to MKS (inplace).
         """
         self.convert_kdata_unit(kdata_unit='m^2')
         self.convert_p_unit(p_unit='Pa')
@@ -180,6 +193,12 @@ class Data_table(object):
                 if an array of to values is given, interpolates only within this array
             log_interp: bool, optional
                 Whether the interpolation is linear in kdata or in log(kdata)
+
+        Returns
+        -------
+            array of shape (logp_array.size, self.Nw (, self.Ng))
+                The interpolated kdata.
+
         """
         if hasattr(logp_array, "__len__"):
             logp_array=np.array(logp_array, dtype=float)
@@ -230,7 +249,8 @@ class Data_table(object):
             # trick for the broadcasting to work whatever the shape of x_array
 
     def remap_logPT(self, logp_array=None, t_array=None, x_array=None):
-        """remap_logPT re-interpolates the kdata on a new temprature and log pressure grid. 
+        """remap_logPT re-interpolates the kdata on a new temperature and log pressure grid
+        (inplace). 
 
         Parameters
         ----------
@@ -281,18 +301,18 @@ class Data_table(object):
         self.Nt      =t_array.size
 
     def pindex(self, p):
-        """Finds the index corresponding to the given pressure p
+        """Finds and returns the index corresponding to the given pressure p
         (units must be the same as the ktable)
         """
         return min(np.searchsorted(self.pgrid,p),self.Np-1)
 
     def tindex(self, t):
-        """Finds the index corresponding to the given temperature t (in K)
+        """Finds and returns the index corresponding to the given temperature t (in K)
         """
         return min(np.searchsorted(self.tgrid,t),self.Nt-1)
 
     def wlindex(self, wl):
-        """Finds the index corresponding to the given wavelength (in microns)
+        """Finds and returns the index corresponding to the given wavelength (in microns)
         """
         return min(np.searchsorted(self.wns,10000./wl),self.Nw-1)-1
         #return min(np.searchsorted(self.wnedges,10000./wl),self.Nw-1)-1
@@ -346,7 +366,6 @@ class Data_table(object):
             ax.plot(self.wns,toplot,**kwarg)
             ax.set_xlabel('Wavenumber (cm$^{-1}$)')
         ax.set_ylabel('Cross section ('+self.kdata_unit+')')
-        ax.grid(True)
         if xscale is not None: ax.set_xscale(xscale)
         if yscale is not None: ax.set_yscale(yscale)
 
@@ -366,7 +385,7 @@ class Data_table(object):
         Returns
         -------
             array
-                The vmr normalized kdata (x_self*self.kdata)
+                The vmr normalized kdata (x_self*self.kdata).
         """
         if x_self is None : return self.kdata
         if isinstance(x_self, (float,int)): return x_self*self.kdata
@@ -444,18 +463,6 @@ class Data_table(object):
         """
         return self.combine_with(other)
 
-    @property
-    def wls(self):
-        """Returns the wavelength array for the bin centers
-        """
-        if self.wns is not None: return 10000./self.wns
-
-    @property
-    def wledges(self):
-        """Returns the wavelength array for the bin edges
-        """
-        if self.wnedges is not None: return 10000./self.wnedges
-
     def __getitem__(self, key):
         """To access the data without typing self.kdata[]
 
@@ -466,7 +473,7 @@ class Data_table(object):
         return self.kdata[key]
 
     def clip_spectral_range(self, wn_range=None, wl_range=None):
-        """Limits the data to the provided spectral range:
+        """Limits the data to the provided spectral range (inplace):
 
            * Wavenumber in cm^-1 if using wn_range argument
            * Wavelength in micron if using wl_range
@@ -498,12 +505,72 @@ class Data_table(object):
         Returns
         -------
             :class:`~exo_k.ktable.Ktable` or :class:`~exo_k.xtable.Xtable` object
-                the binned down table
+                The binned down table.
         """
         res=self.copy()
         res.bin_down(wnedges=wnedges, **kwargs)
         return res
 
+    def change_molecule_name(self, mol_name):
+        """Changes name of the molecule (self.mol attribute).
+
+        Parameters
+        ----------
+            mol_name: str
+                New molecule name
+        """
+        self.mol=mol_name
+
+    @property
+    def molar_mass(self):
+        """Computes molar mass from molecule name
+        """
+        return Molar_mass().fetch(self.mol)
+
+    def write_hdf5_common(self, f, compression="gzip", compression_level=9,
+        p_unit=None):
+        """Method that writes datasets and attributes that are common to
+        X and Ktables. 
+
+        Parameters
+        ----------
+            f: h5py file instance
+                The file to write that is created in daughter classes
+        """
+        dt = h5py.special_dtype(vlen=str)
+        f.create_dataset("DOI", (1,), data=self.DOI, dtype=dt)
+        f.create_dataset("Date_ID", (1,), data=self.Date_ID, dtype=dt)
+        f.create_dataset("mol_name", (1,), data=self.mol, dtype=dt)
+        f.create_dataset("t", data=self.tgrid,
+            compression=compression, compression_opts=compression_level)
+        f["t"].attrs["units"] = 'K'
+
+        if p_unit is not None:
+            conv_factor=u.Unit(self.p_unit).to(u.Unit(p_unit))
+            data_to_write=self.pgrid*conv_factor
+            f.create_dataset("p", data=data_to_write,
+                compression=compression, compression_opts=compression_level)
+            f["p"].attrs["units"] = p_unit
+        else:
+            f.create_dataset("p", data=self.pgrid,
+                compression=compression, compression_opts=compression_level)
+            f["p"].attrs["units"] = self.p_unit
+
+        f.create_dataset("bin_edges", data=self.wnedges,
+            compression=compression, compression_opts=compression_level)
+
+        f.create_dataset("wnrange", data=self.wnrange,
+            compression=compression, compression_opts=compression_level)
+        f.create_dataset("wlrange", data=self.wlrange,
+            compression=compression, compression_opts=compression_level)
+
+        for key,val in ktable_long_name_attributes.items():
+            if key in f: f[key].attrs["long_name"] = val
+        for key in ('bin_edges', 'bin_centers', 'wnrange'):
+            if key in f: f[key].attrs["units"] = self.wn_unit
+        if 'wlrange' in f: f['wlrange'].attrs["units"] = 'micron'
+
+
     def toLogK(self):
         """Changes kdata to log 10.
         """
@@ -519,72 +586,3 @@ class Data_table(object):
             self.logk=False
             self.kdata=np.power(10.,self.kdata)
         return
-
-############# OLD METHODS ################
-
-    def interpolate_kdata2(self, logp_array=None, t_array=None, x_array=None,
-            log_interp=None,wngrid_limit=None):
-        """Deprecated, should NOT be used.
-
-        interpolate_kdata interpolates the kdata at on a given temperature and
-        log pressure profile. 
-
-        Parameters
-        ----------
-            logp_array: array
-                log 10 pressure array to interpolate to
-            t_array: array, same size a logp_array
-                Temperature array to interpolate to
-        If floats are given, they are interpreted as arrays of size 1.
-
-        Parameters
-        ----------
-            x_array: None
-                Dummy argument to be consistent with interpolate_kdata in Ktable5d
-            wngrid_limit: list or array
-                if an array of to values is given, interpolates only within this array
-            log_interp: bool, optional
-                Whether the interpolation is linear in kdata or in log(kdata)
-        """
-        if x_array is not None: print('be careful, providing an x_array is usually for Ktable5d')
-        if hasattr(logp_array, "__len__"):
-            logp_array=np.array(logp_array, dtype=float)
-        else:
-            logp_array=np.array([logp_array], dtype=float)
-        if hasattr(t_array, "__len__"):
-            t_array=np.array(t_array)
-        else:
-            t_array=np.array([t_array])
-        tind,tweight=interp_ind_weights(t_array,self.tgrid)
-        lpind,lpweight=interp_ind_weights(logp_array,self.logpgrid)
-        if wngrid_limit is None:
-            wngrid_filter = slice(None)
-            Nw=self.Nw
-        else:
-            wngrid_limit=np.array(wngrid_limit)
-            wngrid_filter = np.where((self.wnedges > wngrid_limit.min()) & (
-                self.wnedges <= wngrid_limit.max()))[0][:-1]
-            Nw=wngrid_filter.size
-        if self.Ng is None:
-            res=np.zeros((tind.size,Nw))
-        else:
-            res=np.zeros((tind.size,Nw,self.Ng))
-        if log_interp is None: log_interp=self._settings._log_interp
-        if log_interp:
-            for ii in range(tind.size):
-                kc_p1t1=np.log(self.kdata[lpind[ii],tind[ii]][wngrid_filter].ravel())
-                kc_p0t1=np.log(self.kdata[lpind[ii]-1,tind[ii]][wngrid_filter].ravel())
-                kc_p1t0=np.log(self.kdata[lpind[ii],tind[ii]-1][wngrid_filter].ravel())
-                kc_p0t0=np.log(self.kdata[lpind[ii]-1,tind[ii]-1][wngrid_filter].ravel())
-                res[ii]=np.reshape(bilinear_interpolation(kc_p0t0, kc_p1t0, 
-                    kc_p0t1, kc_p1t1, lpweight[ii], tweight[ii]),(Nw,-1)).squeeze()
-            return np.exp(res)
-        else:
-            for ii in range(tind.size):
-                kc_p1t1=self.kdata[lpind[ii],tind[ii]][wngrid_filter].ravel()
-                kc_p0t1=self.kdata[lpind[ii]-1,tind[ii]][wngrid_filter].ravel()
-                kc_p1t0=self.kdata[lpind[ii],tind[ii]-1][wngrid_filter].ravel()
-                kc_p0t0=self.kdata[lpind[ii]-1,tind[ii]-1][wngrid_filter].ravel()
-                res[ii]=np.reshape(bilinear_interpolation(kc_p0t0, kc_p1t0, 
-                    kc_p0t1, kc_p1t1, lpweight[ii], tweight[ii]),(Nw,-1)).squeeze()
-            return res
