@@ -6,8 +6,9 @@ import numpy as np
 from .util.molar_mass import Molar_mass
 from .util.interp import RandOverlap_2_kdata_prof, rm_molec
 from .rayleigh import Rayleigh
+from .util.spectral_object import Spectral_object
 
-class Gas_mix(object):
+class Gas_mix(Spectral_object):
     """Dict-like class to handle gas composition (with background gas) and molar mass.
 
     If `logp_array`, `t_array`, and radiative databases are provided, :any:`cross_section`
@@ -112,7 +113,7 @@ class Gas_mix(object):
         self.composition[self.bg_gas]=1.-other_vmr
 
     def molar_mass(self):
-        """Computes the molar mass of a mix of gases
+        """Computes and returns the molar mass of a mix of gases
 
         Returns
         -------
@@ -175,7 +176,7 @@ class Gas_mix(object):
                 print("""You're being Bad!!! You are trying *NOT* to use MKS units!!!
                 You can convert to mks using convert_to_mks.
                 More generally, you can specify exo_k.Settings().set_mks(True) 
-                to force automatic conversion to mks of all newly loaded data.
+                to force automatic conversion to mks for all newly loaded data.
                 You will have to reload all your data though.
                 (A good thing it does not take so long). """)
                 raise RuntimeError("Bad units in the Kdatabase used with Gas_mix.")
@@ -186,7 +187,7 @@ class Gas_mix(object):
                     You should probably use convert_to_mks().""")
 
     def set_cia_database(self, cia_database=None):
-        """Change the CIA database attached to the current instance of Gas_mix
+        """Changes the CIA database attached to the current instance of Gas_mix
 
         Parameters
         ----------
@@ -204,9 +205,9 @@ class Gas_mix(object):
                 (A good thing it does not take so long). """)
                 raise RuntimeError("Bad units in the CIAdatabase used with Gas_mix.")
 
-    def set_spectral_range(self, wn_range=None, wl_range=None):
-        """Sets the spectral range in which computations will be done by specifying
-        either the wavenumber or the wavelength range.
+    def _compute_spectral_range(self, wn_range=None, wl_range=None):
+        """Converts an unordered spectral range in either wavenumber or wavelength
+        in an ordered wavenumber range.
 
         Parameters
         ----------
@@ -220,20 +221,40 @@ class Gas_mix(object):
                 print('Cannot specify both wl and wn range!')
                 raise RuntimeError()
             else:
-                self._wn_range=np.sort(10000./np.array(wl_range))
+                _wn_range=np.sort(10000./np.array(wl_range))
         else:
             if wn_range is not None:
-                self._wn_range=np.sort(np.array(wn_range))
+                _wn_range=np.sort(np.array(wn_range))
+            else:
+                _wn_range=self._wn_range
+        return _wn_range
 
-    def compute_wn_range_indices(self):
+    def set_spectral_range(self, wn_range=None, wl_range=None):
+        """Sets the default spectral range in which computations will be done by specifying
+        either the wavenumber or the wavelength range.
+
+        Parameters
+        ----------
+            wn_range: list or array of size 2
+                Minimum and maximum wavenumber (in cm^-1).
+            wl_range: list or array of size 2
+                Minimum and maximum wavelength (in micron)
+        """
+        self._wn_range=self._compute_spectral_range(wn_range=wn_range, wl_range=wl_range)
+
+    def _compute_wn_range_indices(self, wn_range=None):
         """Compute the starting and ending indices to be used for current wn_range
         """
-        if self._wn_range is None:
+        if wn_range is None:
+            local_wn_range=self._wn_range
+        else:
+            local_wn_range=wn_range
+        if local_wn_range is None:
             self.iw_min=0
             self.iw_max=self.kdatabase.Nw
         else:
-            self.iw_min, self.iw_max = np.where((self.kdatabase.wnedges > self._wn_range[0]) \
-                & (self.kdatabase.wnedges <= self._wn_range[1]))[0][[0,-1]]
+            self.iw_min, self.iw_max = np.where((self.kdatabase.wnedges > local_wn_range[0]) \
+                & (self.kdatabase.wnedges <= local_wn_range[1]))[0][[0,-1]]
             # to be consistent with interpolate_kdata
 
     def cross_section(self, composition=None, logp_array=None, t_array=None,
@@ -292,8 +313,8 @@ class Gas_mix(object):
              I ll compute opacites with the available ones:""")
         if write>3 : print(mol_to_be_done)
 
-        self.set_spectral_range(wl_range=wl_range, wn_range=wn_range)
-        self.compute_wn_range_indices()
+        local_wn_range=self._compute_spectral_range(wl_range=wl_range, wn_range=wn_range)
+        self._compute_wn_range_indices(wn_range=local_wn_range)
         self.wnedges=np.copy(self.kdatabase.wnedges[self.iw_min:self.iw_max+1])
         self.wns=np.copy(self.kdatabase.wns[self.iw_min:self.iw_max])
         self.Nw=self.wns.size
@@ -302,7 +323,7 @@ class Gas_mix(object):
         first_mol=True
         for mol in mol_to_be_done:
             tmp_kdata=self.kdatabase[mol].interpolate_kdata(logp_array=self.logp_array,
-                t_array=self.t_array, x_array=vmr_prof[mol], wngrid_limit=self._wn_range,
+                t_array=self.t_array, x_array=vmr_prof[mol], wngrid_limit=local_wn_range,
                 **kwargs)
             if first_mol:
                 kdata_array=tmp_kdata
@@ -320,7 +341,7 @@ class Gas_mix(object):
             cont_sig=np.zeros((self.Narray,self.Nw))
             if self.cia_database is not None:
                 cont_sig+=self.cia_database.cia_cross_section(self.logp_array,
-                    self.t_array, vmr_prof, wngrid_limit=self._wn_range)
+                    self.t_array, vmr_prof, wngrid_limit=local_wn_range)
             if rayleigh:
                 if cst_prof:
                     cont_sig+=Rayleigh().sigma(self.wns, self.composition)
@@ -389,18 +410,6 @@ class Gas_mix(object):
         res.iw_min=self.iw_min
         res.iw_max=self.iw_max
 
-    @property
-    def wls(self):
-        """Returns the wavelength array for the bin centers
-        """
-        if self.wns is not None: return 10000./self.wns
-
-    @property
-    def wledges(self):
-        """Returns the wavelength array for the bin edges
-        """
-        if self.wnedges is not None: return 10000./self.wnedges
-
     def mix_with(self, other_gas, vmr_other_gas):
         """Mix with other Gas_mix.
         """
