@@ -1,27 +1,30 @@
 # -*- coding: utf-8 -*-
 """
 @author: jeremy leconte
-Doc can be created with "pydoc -w corrk_lib"
 """
 import os
 from math import log10
 import time
-import pickle
-import h5py
 import numpy as np
 import astropy.units as u
-from .data_table import Data_table
+from .ktable_io import Ktable_io
 from .ktable5d import Ktable5d
-from .util.interp import rm_molec, interp_ind_weights, rebin_ind_weights, rebin, is_sorted, \
-        gauss_legendre, spectrum_to_kdist, kdata_conv_loop, bin_down_corrk_numba
+from .util.interp import rebin_ind_weights, rebin, is_sorted, \
+        gauss_legendre, split_gauss_legendre, spectrum_to_kdist, \
+        kdata_conv_loop, bin_down_corrk_numba, g_sample_4d
 from .hires_spectrum import Hires_spectrum
-from .util.filenames import create_fname_grid_Kspectrum_LMDZ, select_kwargs, read_nemesis_binary
-from .util.cst import KBOLTZ, nemesis_hitran_id_numbers
+from .util.filenames import create_fname_grid_Kspectrum_LMDZ, select_kwargs
+from .util.cst import KBOLTZ
 
 
-class Ktable(Data_table):
+class Ktable(Ktable_io):
     """A class that handles 4D tables of k-coefficients.
-    Based on the Data_table class that handles basic operations common to Xtable.
+    
+    Based on the :class:`~exo_k.data_table.Data_table` class
+    that handles basic operations common to Xtable.
+
+    Based on the :class:`~exo_k.ktable_io.Ktable_io` class
+    that incorporates all io routines (read/write_xxx for all the xxx supported formats).
     """
         
     def __init__(self, *filename_filters, filename=None, xtable=None, path=None,
@@ -97,18 +100,22 @@ class Ktable(Data_table):
         elif filename_filters or (mol is not None and (xtable is None and path is None)): 
         # a none empty sequence returns a True in a conditional statement
             self.filename=self._settings.list_files(*filename_filters, molecule = mol,
-                only_one=True, search_path=search_path)[0]
+                only_one=True, search_path=search_path, path_type='ktable')[0]
 
         if self.filename is not None:
             if self.filename.lower().endswith('pickle'):
-                self.read_pickle(filename=self.filename, mol=mol)
+                self.read_pickle(filename=self.filename)
             elif self.filename.lower().endswith(('.hdf5', '.h5')):
                 self.read_hdf5(filename=self.filename, mol=mol)
             elif self.filename.lower().endswith('.kta'):
                 self.read_nemesis(filename=self.filename, mol=mol)
+            elif self.filename.lower().endswith('ktable.exorem.txt'):
+                self.read_exorem(filename=self.filename, mol=mol)
+            elif self.filename.lower().endswith('.fits'):
+                self.read_arcis(filename=self.filename, mol=mol)
             else:
                 raise NotImplementedError("""Requested format not recognized.
-            Should end with .pickle, .hdf5, .h5, or .kta""")
+            Should end with .pickle, .hdf5, .h5, .fits, ktable.exorem.txt, or .kta""")
         elif xtable is not None:
             self.xtable_to_ktable(xtable=xtable, **kwargs)
         elif path is not None:
@@ -123,343 +130,16 @@ class Ktable(Data_table):
             kdata_unit=kdata_unit, file_kdata_unit=file_kdata_unit,
             remove_zeros=remove_zeros)
 
-
     @property
     def shape(self):
         """Returns the shape of self.kdata
         """
         return np.array([self.Np,self.Nt,self.Nw,self.Ng])
 
-    def read_pickle(self, filename=None, mol=None):
-        """Initializes k coeff table and supporting data from an Exomol pickle file
-
-        Parameters
-        ----------
-            filename: str
-                Name of the input pickle file
-            mol: str, optional
-                Force the name of the molecule
-        """
-        if filename is None: raise RuntimeError("You should provide an input pickle filename")
-        pickle_file=open(filename,'rb')
-        raw=pickle.load(pickle_file, encoding='latin1')
-        pickle_file.close()
-        
-        if mol is not None: 
-            self.mol=mol
-        else:
-            self.mol=raw['name']
-            if self.mol=='H2OP': self.mol='H2O'
-
-        self.pgrid=raw['p']
-        self.logpgrid=np.log10(self.pgrid)
-        self.tgrid=raw['t']
-        self.wns=raw['bin_centers']
-        self.wnedges=raw['bin_edges']
-        if 'weights' in raw.keys():
-            self.weights=raw['weights']
-        else:
-            raise RuntimeError('No weights keyword. This file is probably a cross section file.')
-        self.ggrid=raw['samples']
-        self.gedges=np.insert(np.cumsum(self.weights),0,0.)
-        self.kdata=raw['kcoeff']
-    
-        if 'p_unit' in raw.keys():
-            self.p_unit=raw['p_unit']
-        else:
-            self.p_unit='bar'
-
-        if 'kdata_unit' in raw.keys():
-            self.kdata_unit=raw['kdata_unit']
-        else:
-            self.kdata_unit='cm^2/molec'
-
-        if 'wn_unit' in raw.keys(): self.wn_unit=raw['wn_unit']
-
-        self.Np,self.Nt,self.Nw,self.Ng=self.kdata.shape
-
-    def write_pickle(self, filename):
-        """Saves data in a pickle format
-
-        Parameters
-        ----------
-            filename: str
-                Name of the file to be created and saved
-        """
-        fullfilename=filename
-        if not filename.lower().endswith('.pickle'): fullfilename=filename+'.pickle'
-        pickle_file=open(fullfilename,'wb')
-        dictout={'name':self.mol,
-                 'p':self.pgrid,
-                 'p_unit':self.p_unit,
-                 't':self.tgrid,
-                 'bin_centers':self.wns,
-                 'bin_edges':self.wnedges,
-                 'wn_unit':self.wn_unit,
-                 'weights':self.weights,
-                 'samples':self.ggrid,
-                 'kcoeff':self.kdata,
-                 'kdata_unit':self.kdata_unit}
-        #print(dictout)
-        pickle.dump(dictout,pickle_file,protocol=-1)
-        pickle_file.close()
-
-    def read_hdf5(self, filename=None, mol=None):
-        """Initializes k coeff table and supporting data from an Exomol hdf5 file
-
-        Parameters
-        ----------
-            file : str
-                Name of the input hdf5 file
-        """
-        if (filename is None or not filename.lower().endswith(('.hdf5', '.h5'))):
-            raise RuntimeError("You should provide an input hdf5 file")
-        f = h5py.File(filename, 'r')
-        if 'mol_name' in f:
-            self.mol=f['mol_name'][()]
-        elif 'mol_name' in f.attrs:
-            self.mol=f.attrs['mol_name']
-        else:
-            self.mol=os.path.basename(filename).split(self._settings._delimiter)[0]
-        if isinstance(self.mol, np.ndarray): self.mol=self.mol[0]
-        if mol is not None:
-            self.mol=mol
-        self.wns=f['bin_centers'][...]
-        self.wnedges=f['bin_edges'][...]
-        if 'units' in f['bin_edges'].attrs:
-            self.wn_unit=f['bin_edges'].attrs['units']
-        else:
-            if 'units' in f['bin_centers'].attrs:
-                self.wn_unit=f['bin_centers'].attrs['units']
-        self.kdata=f['kcoeff'][...]
-        self.kdata_unit=f['kcoeff'].attrs['units']
-        self.tgrid=f['t'][...]
-        self.pgrid=f['p'][...]
-        self.logpgrid=np.log10(self.pgrid)
-        self.p_unit=f['p'].attrs['units']
-        if 'weights' in f.keys():
-            self.weights=f['weights'][...]
-        else:
-            raise RuntimeError('No weights keyword. This file is probably a cross section file.')
-        self.ggrid=f['samples'][...]
-        self.gedges=np.insert(np.cumsum(self.weights),0,0.)
-        self.logk=False
-        f.close()  
-        self.Np,self.Nt,self.Nw,self.Ng=self.kdata.shape
-
-    def write_hdf5(self, filename):
-        """Saves data in a hdf5 format
-
-        Parameters
-        ----------
-            filename: str
-                Name of the file to be created and saved
-        """
-        fullfilename=filename
-        if not filename.lower().endswith(('.hdf5', '.h5')):
-            fullfilename=filename+'.hdf5'
-        compression="gzip"
-        f = h5py.File(fullfilename, 'w')
-        f.create_dataset("mol_name", data=self.mol)
-        f.create_dataset("p", data=self.pgrid,compression=compression)
-        f["p"].attrs["units"] = self.p_unit
-        f.create_dataset("t", data=self.tgrid,compression=compression)
-        f.create_dataset("kcoeff", data=self.kdata,compression=compression)
-        f["kcoeff"].attrs["units"] = self.kdata_unit
-        f.create_dataset("samples", data=self.ggrid,compression=compression)
-        f.create_dataset("weights", data=self.weights,compression=compression)
-        f.create_dataset("bin_edges", data=self.wnedges,compression=compression)
-        f.create_dataset("bin_centers", data=self.wns,compression=compression)
-        f["bin_edges"].attrs["units"] = self.wn_unit
-        f.close()    
-
-    def read_LMDZ(self, path=None, res=None, band=None, mol=None):
-        """Initializes k coeff table and supporting data from a .dat file in a gcm friendly format.
-        
-        Units are assumed to be cm^2 for kdata and mbar for pressure. 
-
-        Parameters
-        ----------
-            path: str
-                Name of the directory with the various input files
-            res: str
-                "IRxVI" where IR and VI are the numbers of bands
-                in the infrared and visible of the k table to load.
-            band: str
-                "IR" or "VI" to specify which band to load.
-            mol: str
-                Name of the molecule to be saved in the Ktable object. 
-        """        
-        if (path is None) or (res is None): \
-            raise TypeError("You should provide an input directory name and a resolution")
-
-        self.filename=path
-        if mol is not None:
-            self.mol=mol
-        else:
-            self.mol=os.path.basename(self.filename).split(self._settings._delimiter)[0]
-
-        self.weights=np.loadtxt(os.path.join(path,'g.dat'),skiprows=1)[:-1]
-            # we remove the last point that is always zero.
-            # in the gcm this last point is intended to take care of continuum
-        self.Ng=self.weights.size
-        self.gedges=np.insert(np.cumsum(self.weights),0,0.)
-        self.ggrid=(self.gedges[1:]+self.gedges[:-1])*0.5
-
-        self.p_unit='mbar'
-        self.logpgrid=np.loadtxt(os.path.join(path,'p.dat'),skiprows=1)*1.
-        self.Np=self.logpgrid.size
-        self.pgrid=10**self.logpgrid
-
-        self.tgrid=np.loadtxt(os.path.join(path,'T.dat'),skiprows=1)
-        self.Nt=self.tgrid.size
-
-        if band is None:
-            raw=np.loadtxt(os.path.join(path,res,'narrowbands.in'), skiprows=1, unpack=True)
-        else:
-            raw=np.loadtxt(os.path.join(path,res,'narrowbands_'+band+'.in'), \
-                skiprows=1, unpack=True)
-        self.wnedges=np.append(raw[0],raw[1,-1])
-        self.wns=(self.wnedges[1:]+self.wnedges[:-1])*0.5
-        self.Nw=self.wns.size
-        
-        self.kdata_unit='cm^2/molecule'
-        if band is None:
-            file_to_load=os.path.join(path,res,'corrk_gcm.dat')
-        else:
-            file_to_load=os.path.join(path,res,'corrk_gcm_'+band+'.dat')        
-        tmp=np.loadtxt(file_to_load).reshape((self.Nt,self.Np,self.Nw,self.Ng+1),order='F')
-        self.kdata=tmp[:,:,:,:-1].transpose((1,0,2,3)) 
-        # also removing the last g point which is equal to 0.
-        self.logk=False        
-        return None
-
-    def write_LMDZ(self, path, band='IR', fmt='%22.15e', write_only_metadata=False):
-        """Saves data in a LMDZ friendly format.
-        
-        The gcm requires p in mbar and kdata in cm^2/molec.
-        The conversion is done automatically.
-
-        Parameters
-        ----------
-            path: str
-                Name of the directory to be created and saved,
-                the one that will contain all the necessary files
-            band: str
-                The band you are computing: 'IR' or 'VI'
-            fmt: str
-                Fortran format for the corrk file. 
-        """
-        try:
-            os.mkdir(path)
-        except FileExistsError:
-            print('Directory was already there '+path)
-        file = open(os.path.join(path,'p.dat'), "w")
-        file.write(str(self.Np)+'\n')
-        lp_to_write=self.logpgrid+np.log10(u.Unit(self.p_unit).to(u.Unit('mbar')))
-        for lp in lp_to_write:
-            file.write(str(lp)+'\n')
-        file.close()
-
-        file = open(os.path.join(path,'T.dat'), "w")
-        file.write(str(self.Nt)+'\n')
-        for t in self.tgrid:
-            file.write(str(t)+'\n')
-        file.close()
-
-        file = open(os.path.join(path,'g.dat'), "w")
-        file.write(str(self.Ng+1)+'\n')
-        for g in self.weights:
-            file.write(str(g)+'\n')
-        file.write(str(0.)+'\n')
-        file.close()
-
-        dirname=os.path.join(path,band+str(self.Nw))
-        try:
-            os.mkdir(dirname)
-        except FileExistsError:
-            print('Directory was already there: '+dirname)
-
-        file = open(os.path.join(dirname,'narrowbands_'+band+'.in'), "w")
-        file.write(str(self.Nw)+'\n')
-        for iw in range(self.Nw):
-            file.write(str(self.wnedges[iw])+' '+str(self.wnedges[iw+1])+'\n')
-        file.close()
-
-        if not write_only_metadata:
-            #file = open(dirname+'/corrk_gcm_IR.in', "w")
-            data_to_write=self.kdata.transpose((1,0,2,3)).flatten(order='F')
-            data_to_write=data_to_write*u.Unit(rm_molec(self.kdata_unit)).to(u.Unit('cm^2'))
-            data_to_write=np.append(data_to_write, \
-                np.zeros(self.Np*self.Nt*self.Nw)).reshape((1,self.Np*self.Nt*self.Nw*(self.Ng+1)))
-            np.savetxt(os.path.join(dirname,'corrk_gcm_'+band+'.dat'),data_to_write,fmt=fmt)
-
-    def read_nemesis(self, filename=None, mol=None):
-        """Initializes k coeff table and supporting data from a Nemesis binary file (.kta)
-
-        Parameters
-        ----------
-            file: str
-                Name of the input Nemesis binary file.
-            mol: str, optional
-                Name of the molecule.
-        """
-        if (filename is None or not filename.lower().endswith('.kta')):
-            raise RuntimeError("You should provide an input nemesis (.kta) file")
-        
-        if mol is not None:
-            self.mol=mol
-        else:
-            self.mol=os.path.basename(self.filename).split(self._settings._delimiter)[0]
-
-        self.Np, self.Nt, self.Nw, self.Ng, \
-            self.pgrid, self.tgrid, self.wns, \
-            self.ggrid, self.weights, self.kdata = read_nemesis_binary(filename)
-
-        self.logpgrid=np.log10(self.pgrid)
-        self.wnedges=np.concatenate(  \
-            ([self.wns[0]],(self.wns[:-1]+self.wns[1:])*0.5,[self.wns[-1]]))
-        self.gedges=np.insert(np.cumsum(self.weights),0,0.)
-        self.p_unit='bar'
-        self.kdata_unit='cm^2/molecule'
-
-    def write_nemesis(self, filename):
-        """Saves data in a nemesis format.
-
-        base on a routine provided by K. Chubb.
-
-        Parameters
-        ----------
-            filename: str
-                Name of the file to be created and saved
-        """
-        int_format = np.int32
-        float_format = np.float32
-        with open(filename,'wb') as o:
-            o.write(int_format(11+2*self.Ng+2+self.Np+self.Nt+self.Nw).tostring())
-            #int for irec0 (5221)
-            o.write(int_format(self.Nw).tostring()) #int
-            o.write(float_format(self.wls[-1]).tostring()) #float for VMIN (0.30015)
-            o.write(float_format(-1.).tostring()) #float
-            o.write(float_format(0.).tostring()) #float for FWHM
-            o.write(int_format(self.Np).tostring())
-            o.write(int_format(self.Nt).tostring())
-            o.write(int_format(self.Ng).tostring())
-            o.write(int_format(nemesis_hitran_id_numbers[self.mol]).tostring()) #IDGAS1 FROM HITRAN
-            o.write(int_format(0).tostring())
-            o.write(float_format(self.ggrid).tostring())
-            o.write(float_format(self.weights).tostring())
-            o.write(float_format(0.).tostring()) #float for 0
-            o.write(float_format(0.).tostring()) #float for 0
-            o.write(float_format(self.pgrid).tostring())
-            o.write(float_format(self.tgrid).tostring())
-            o.write(float_format(self.wls[::-1]).tostring())
-            o.write(float_format(self.kdata[:,:,::-1,:].transpose(2,0,1,3)*1.e20).tostring())
-
     def xtable_to_ktable(self, xtable=None, wnedges=None, weights=None, ggrid=None,
-        quad='legendre', order=20, mid_dw=True, write=0):
+        quad='legendre', order=20, g_split=0.9, mid_dw=True, write=0, remove_zeros=False):
         """Fills the :class:`~exo_k.ktable.Ktable` object with a k-coeff table computed
-        from a :class:`~exo_k.xtable.Xtable` object.
+        from a :class:`~exo_k.xtable.Xtable` object (inplace).
 
         The p and kcorr units are inherited from the :class:`~exo_k.xtable.Xtable` object.
 
@@ -475,9 +155,13 @@ class Ktable(Data_table):
             weights: array, optional
                 If weights are provided, they are used instead of the legendre quadrature. 
             quad: string, optional
-                Type of quadrature used. Default is 'legendre'
+                Type of quadrature used. Default is 'legendre'.
+                Also available: 'split-legendre' which uses half quadrature
+                points between 0. and `g_split` and half between `g_split` and 1.
             order: Integer, optional
                 Order of the Gauss legendre quadrature used. Default is 20.
+            g_split: float, optional
+                Used only if quad='split-legendre'. See above.
             mid_dw: boolean, optional
 
                 * If True, the Xsec values in the high resolution xtable data are assumed to
@@ -502,11 +186,15 @@ class Ktable(Data_table):
                 self.ggrid=np.array(ggrid)
             else:
                 self.ggrid=(self.gedges[1:]+self.gedges[:-1])*0.5
+            self.sampling_method='custom'
         else:
             if quad=='legendre':
                 self.weights,self.ggrid,self.gedges=gauss_legendre(order)
+            elif quad=='split-legendre':
+                self.weights,self.ggrid,self.gedges=split_gauss_legendre(order, g_split)
             else:
                 raise NotImplementedError("Type of quadrature (quad keyword) not known.")
+            self.sampling_method=quad
         self.Ng=self.weights.size
         if write >6 : print(self.weights,self.ggrid,self.gedges)
         self.logk=False
@@ -528,16 +216,18 @@ class Ktable(Data_table):
             #print(wn_hr[[0,-1]])
             #print(self.wnedges[[0,-1]])
             self.kdata[iP,iT]=spectrum_to_kdist(k_hr,wn_hr,dwn_hr,self.wnedges,self.ggrid)
+        if remove_zeros : self.remove_zeros(deltalog_min_value=10.)
 
     def hires_to_ktable(self, path=None, filename_grid=None,
         logpgrid=None, tgrid=None, wnedges=None,
-        quad='legendre', order=20, weights=None, ggrid=None,
+        quad='legendre', order=20, g_split=0.9, weights=None, ggrid=None,
         mid_dw=True, write=0, mol=None,
         grid_p_unit='Pa', p_unit='unspecified',
         kdata_unit='unspecified', file_kdata_unit='unspecified',
+        remove_zeros=False,
         **kwargs):
         """Computes a k coeff table from :class:`~exo_k.util.hires_spectrum.Hires_spectrum`
-        objects.
+        objects (inplace).
 
         .. warning::
             By default, log pressures are specified in Pa in logpgrid!!! If you want
@@ -565,10 +255,14 @@ class Ktable(Data_table):
         ----------------
             weights: array, optional
                 If weights are provided, they are used instead of the legendre quadrature. 
-            quad : string, optional
-                Type of quadrature used. Default is 'legendre'
-            order : Integer, optional
+            quad: string, optional
+                Type of quadrature used. Default is 'legendre'.
+                Also available: 'split-legendre' which uses half quadrature
+                points between 0. and `g_split` and half between `g_split` and 1.
+            order: Integer, optional
                 Order of the Gauss legendre quadrature used. Default is 20.
+            g_split: float, optional
+                Used only if quad='split-legendre'. See above.
             mid_dw: boolean, optional
                 * If True, the Xsec values in the high resolution xsec data are assumed to
                   cover a spectral interval that is centered around
@@ -609,11 +303,15 @@ class Ktable(Data_table):
                 self.ggrid=np.array(ggrid)
             else:
                 self.ggrid=(self.gedges[1:]+self.gedges[:-1])*0.5
+            self.sampling_method='custom'
         else:
             if quad=='legendre':
                 self.weights,self.ggrid,self.gedges=gauss_legendre(order)
+            elif quad=='split-legendre':
+                self.weights,self.ggrid,self.gedges=split_gauss_legendre(order, g_split)
             else:
                 raise NotImplementedError("Type of quadrature (quad keyword) not known.")
+            self.sampling_method=quad
         self.Ng=self.weights.size
 
         conversion_factor=u.Unit(grid_p_unit).to(u.Unit('Pa'))
@@ -678,10 +376,12 @@ class Ktable(Data_table):
         #        file_kdata_unit into account)
         #  to the desired unit.
         self.convert_p_unit(p_unit=p_unit)
+        if remove_zeros : self.remove_zeros(deltalog_min_value=10.)
+
 
 
     def copy(self, cp_kdata=True, ktab5d=False):
-        """Creates a new instance of :class:`Ktable` object and (deep) copies data into it
+        """Creates a new instance of :class:`Ktable` object and (deep) copies data into it.
 
         Parameters
         ----------
@@ -754,7 +454,6 @@ class Ktable(Data_table):
             ax.plot(self.ggrid,toplot,**kwarg)
             ax.set_xlabel('g')
         ax.set_ylabel('Cross section ('+self.kdata_unit+')')
-        ax.grid(True)
         if yscale is not None: ax.set_yscale(yscale)
 
     def __repr__(self):
@@ -835,7 +534,7 @@ class Ktable(Data_table):
         
     def bin_down(self, wnedges=None, weights=None, ggrid=None,
         remove_zeros=False, num=300, use_rebin=False, write=0):
-        """Method to bin down a kcoeff table to a new grid of wavenumbers
+        """Method to bin down a kcoeff table to a new grid of wavenumbers (inplace).
 
         Parameters
         ----------
@@ -879,51 +578,23 @@ class Ktable(Data_table):
         self.Nw=self.wns.size
         if remove_zeros : self.remove_zeros(deltalog_min_value=10.)
 
+    def remap_g(self, ggrid=None, weights=None):
+        """Method to resample a kcoeff table to a new g grid (inplace).
 
-########### OBSOLETE METHODS #################
-
-    def remap_logPT_grid(self, logp_array=None, t_array=None):
-        """Obsolete method. Replaced by remap_logPT in the parent class: Data_table.
+        Parameters
+        ----------
+            ggrid: array
+                New grid of abcissas for quadrature
+            weights: array
+                New grid of weights
         """
-        res=Ktable()
-        res.mol     =self.mol
-        res.logpgrid=logp_array
-        res.pgrid   =10**res.logpgrid
-        res.tgrid   =t_array
-        res.wns     =self.wns
-        res.wnedges =self.wnedges
-        res.weights =self.weights
-        res.ggrid   =self.ggrid
-        res.gedges  =self.gedges
-        res.Np      =logp_array.shape
-        res.Nt      =t_array.shape
-        res.Nw      =self.Nw
-        res.Ng      =self.Ng
-        res.logk    =self.logk
-        res.p_unit  =self.p_unit
-        res.kdata_unit=self.kdata_unit
-        #res.kdata=np.zeros((res.Np,res.Nt,res.Nw ,res.Ng))
-        tind,tweight=interp_ind_weights(t_array,self.tgrid)
-        lpind,lpweight=interp_ind_weights(logp_array,self.logpgrid)
-        #print(tind,tweight)
-        #print(lpind,lpweight)
-        lpindextended=lpind.reshape((lpind.size,1))
-        tw=tweight.reshape((1,tweight.size,1,1))    
-        pw=lpweight.reshape((lpweight.size,1,1,1))
-        # trick to broadcast over Nw and Ng a few lines below
-        kc_p1t1=self.kdata[lpindextended,tind]
-        kc_p0t1=self.kdata[lpindextended-1,tind]
-        kc_p1t0=self.kdata[lpindextended,tind-1]
-        kc_p0t0=self.kdata[lpindextended-1,tind-1]
-        # kdata_tmp=kc_p1t1*pw*tw+kc_p0t1*(1.-pw)*tw+kc_p1t0*pw*(1.-tw)+kc_p0t0*(1.-pw)*(1.-tw)
-        kdata_tmp=  np.log10(kc_p1t1)*pw*tw          \
-                        +np.log10(kc_p0t1)*(1.-pw)*tw \
-                        +np.log10(kc_p1t0)*pw*(1.-tw) \
-                        +np.log10(kc_p0t0)*(1.-pw)*(1.-tw)
-        res.kdata=10**kdata_tmp
-        #kdata_tmp=  (kc_p1t1)*pw*tw          \
-        #                +(kc_p0t1)*(1.-pw)*tw \
-        #                +(kc_p1t0)*pw*(1.-tw) \
-        #                +(kc_p0t0)*(1.-pw)*(1.-tw)
-        #res.kdata=kdata_tmp
-        return res
+        weights=np.array(weights)
+        ggrid=np.array(ggrid)
+        self.Ng=weights.size
+        newkdata=np.zeros((self.Np, self.Nt, self.Nw, self.Ng))
+        g_sample_4d(ggrid, newkdata, self.ggrid, self.kdata)
+        self.kdata=newkdata
+        self.ggrid=ggrid
+        self.weights=weights
+        self.gedges=np.concatenate(([0],np.cumsum(self.weights)))
+
