@@ -10,7 +10,8 @@ import astropy.units as u
 from scipy.interpolate import RegularGridInterpolator
 from .data_table import Data_table
 from .util.interp import rm_molec, rebin_ind_weights, \
-        gauss_legendre, spectrum_to_kdist, is_sorted, bin_down_corrk_numba
+        gauss_legendre, split_gauss_legendre, spectrum_to_kdist, \
+        is_sorted, bin_down_corrk_numba, g_sample_5d
 from .util.cst import KBOLTZ
 from .hires_spectrum import Hires_spectrum
 from .util.filenames import create_fname_grid_Kspectrum_LMDZ, select_kwargs
@@ -55,13 +56,13 @@ class Ktable5d(Data_table):
         elif filename_filters or (mol is not None and path is None): 
         # a none empty sequence returns a True in a conditional statement
             self.filename=self._settings.list_files(*filename_filters, molecule = mol,
-                only_one=True, search_path=search_path)[0]
+                only_one=True, search_path=search_path, path_type='ktable')[0]
         if self.filename is not None:
             if self.filename.lower().endswith(('.hdf5', '.h5')):
                 self.read_hdf5(filename=self.filename, mol=mol)
             else:
                 raise NotImplementedError( \
-                    'Requested format not recognized. Should end with .pickle, .hdf5, or .h5')
+                    'Requested format not recognized. Should end with .hdf5 or .h5')
         elif path is not None:
             self.read_LMDZ(path=path, mol=mol, **kwargs)
         else:                  #if there is no input file, just create an empty object 
@@ -95,40 +96,47 @@ class Ktable5d(Data_table):
         """
         if (filename is None or not filename.lower().endswith(('.hdf5', '.h5'))):
             raise RuntimeError("You should provide an input hdf5 file")
-        f = h5py.File(filename, 'r')
-        if 'mol_name' in f:
-            self.mol=f['mol_name'][()]
-        elif 'mol_name' in f.attrs:
-            self.mol=f.attrs['mol_name']
-        else:
-            self.mol=os.path.basename(filename).split(self._settings._delimiter)[0]
-        if isinstance(self.mol, np.ndarray): self.mol=self.mol[0]
-        if mol is not None: self.mol=mol
-        self.wns=f['bin_centers'][...]
-        self.wnedges=f['bin_edges'][...]
-        if 'units' in f['bin_edges'].attrs:
-            self.wn_unit=f['bin_edges'].attrs['units']
-        else:
-            if 'units' in f['bin_centers'].attrs:
-                self.wn_unit=f['bin_centers'].attrs['units']
-        self.kdata=f['kcoeff'][...]
-        self.kdata_unit=f['kcoeff'].attrs['units']
-        self.tgrid=f['t'][...]
-        self.pgrid=f['p'][...]
-        self.logpgrid=np.log10(self.pgrid)
-        self.p_unit=f['p'].attrs['units']
-        self.xgrid=f['x'][...]
-        if 'weights' in f.keys():
-            self.weights=f['weights'][...]
-        else:
-            raise RuntimeError('No weights keyword. This file is probably a cross section file.')
-        self.ggrid=f['samples'][...]
-        self.gedges=np.insert(np.cumsum(self.weights),0,0.)
-        self.logk=False
+        with h5py.File(filename, 'r') as f:
+            if 'mol_name' in f:
+                self.mol=f['mol_name'][()]
+            elif 'mol_name' in f.attrs:
+                self.mol=f.attrs['mol_name']
+            else:
+                if mol is not None:
+                    self.mol=mol
+                else:
+                    self.mol=os.path.basename(filename).split(self._settings._delimiter)[0]
+            if isinstance(self.mol, np.ndarray): self.mol=self.mol[0]
+            if 'method' in f:
+                self.sampling_method=f['method'][()][0]
+            if 'DOI' in f:
+                self.DOI=f['DOI'][()][0]
+            self.wns=f['bin_centers'][...]
+            self.wnedges=f['bin_edges'][...]
+            if 'units' in f['bin_edges'].attrs:
+                self.wn_unit=f['bin_edges'].attrs['units']
+            else:
+                if 'units' in f['bin_centers'].attrs:
+                    self.wn_unit=f['bin_centers'].attrs['units']
+            self.kdata=f['kcoeff'][...]
+            self.kdata_unit=f['kcoeff'].attrs['units']
+            self.tgrid=f['t'][...]
+            self.pgrid=f['p'][...]
+            self.logpgrid=np.log10(self.pgrid)
+            self.p_unit=f['p'].attrs['units']
+            self.xgrid=f['x'][...]
+            if 'weights' in f.keys():
+                self.weights=f['weights'][...]
+            else:
+                raise RuntimeError('No weights keyword. This file is probably a cross section file.')
+            self.ggrid=f['samples'][...]
+            self.gedges=np.insert(np.cumsum(self.weights),0,0.)
+            self.logk=False
         f.close()  
         self.Np,self.Nt,self.Nx,self.Nw,self.Ng=self.kdata.shape
 
-    def write_hdf5(self, filename):
+    def write_hdf5(self, filename, compression="gzip", compression_level=9,
+        kdata_unit=None, p_unit=None):
         """Saves data in a hdf5 format
 
         Parameters
@@ -136,24 +144,36 @@ class Ktable5d(Data_table):
             filename: str
                 Name of the file to be created and saved
         """
+        dt = h5py.special_dtype(vlen=str)
         fullfilename=filename
         if not filename.lower().endswith(('.hdf5', '.h5')):
-            fullfilename=filename+'.hdf5'
-        compression="gzip"
-        f = h5py.File(fullfilename, 'w')
-        f.create_dataset("mol_name", data=self.mol)
-        f.create_dataset("p", data=self.pgrid,compression=compression)
-        f["p"].attrs["units"] = self.p_unit
-        f.create_dataset("t", data=self.tgrid,compression=compression)
-        f.create_dataset("x", data=self.xgrid,compression=compression)
-        f.create_dataset("kcoeff", data=self.kdata,compression=compression)
-        f["kcoeff"].attrs["units"] = self.kdata_unit
-        f.create_dataset("samples", data=self.ggrid,compression=compression)
-        f.create_dataset("weights", data=self.weights,compression=compression)
-        f.create_dataset("bin_edges", data=self.wnedges,compression=compression)
-        f.create_dataset("bin_centers", data=self.wns,compression=compression)
-        f["bin_edges"].attrs["units"] = self.wn_unit
-        f.close()    
+            fullfilename=filename+'.h5'
+        with h5py.File(fullfilename, 'w') as f:
+            f.create_dataset("x", data=self.xgrid, compression=compression,
+                compression_opts=compression_level)
+            f["x"].attrs["units"] = 'dimentionless'
+            if kdata_unit is not None:
+                conv_factor=u.Unit(rm_molec(self.kdata_unit)).to(u.Unit(rm_molec(kdata_unit)))
+                data_to_write=self.kdata*conv_factor
+                f.create_dataset("kcoeff", data=data_to_write,
+                    compression=compression, compression_opts=compression_level)
+                f["kcoeff"].attrs["units"] = kdata_unit
+            else:
+                f.create_dataset("kcoeff", data=self.kdata,
+                    compression=compression, compression_opts=compression_level)
+                f["kcoeff"].attrs["units"] = self.kdata_unit
+            f.create_dataset("method", (1,), data=self.sampling_method, dtype=dt)
+            f.create_dataset("samples", data=self.ggrid,
+                compression=compression, compression_opts=compression_level)
+            f.create_dataset("weights", data=self.weights,
+                compression=compression, compression_opts=compression_level)
+            f.create_dataset("ngauss", data=self.Ng)
+            f.create_dataset("bin_centers", data=self.wns,
+                compression=compression, compression_opts=compression_level)
+
+            # where most of the data is actually written
+            self.write_hdf5_common(f, compression=compression, compression_level=compression_level,
+            p_unit=p_unit)
 
     def read_LMDZ(self, path=None, res=None, band=None, mol=None):
         """Initializes k coeff table and supporting data from a .dat file
@@ -280,10 +300,11 @@ class Ktable5d(Data_table):
 
     def hires_to_ktable(self, path=None, filename_grid=None,
         logpgrid=None, tgrid=None, xgrid=None, wnedges=None,
-        quad='legendre', order=20, weights=None, ggrid=None,
+        quad='legendre', order=20, g_split=0.9, weights=None, ggrid=None,
         mid_dw=True, write=0, mol=None,
         grid_p_unit='Pa', p_unit='unspecified',
         kdata_unit='unspecified', file_kdata_unit='unspecified',
+        remove_zeros=False,
         **kwargs):
         """Computes a k coeff table from :class:`~exo_k.util.hires_spectrum.Hires_spectrum`
         objects.
@@ -317,11 +338,15 @@ class Ktable5d(Data_table):
                 self.ggrid=np.array(ggrid)
             else:
                 self.ggrid=(self.gedges[1:]+self.gedges[:-1])*0.5
+            self.sampling_method='custom'
         else:
             if quad=='legendre':
                 self.weights,self.ggrid,self.gedges=gauss_legendre(order)
+            elif quad=='split-legendre':
+                self.weights,self.ggrid,self.gedges=split_gauss_legendre(order, g_split)
             else:
                 raise NotImplementedError("Type of quadrature (quad keyword) not known.")
+            self.sampling_method=quad
         self.Ng=self.weights.size
 
         conversion_factor=u.Unit(grid_p_unit).to(u.Unit('Pa'))
@@ -391,9 +416,12 @@ class Ktable5d(Data_table):
         #        file_kdata_unit into account)
         #  to the desired unit.
         self.convert_p_unit(p_unit=p_unit)
+        if remove_zeros : self.remove_zeros(deltalog_min_value=10.)
+        self.setup_interpolation()
 
     def setup_interpolation(self, log_interp=None):
-        """Creates interpolating functions to be called later on.
+        """Creates interpolating functions to be called later on. and loads
+        it as attribute (inplace).
         """
         if log_interp is None: log_interp=self._settings._log_interp
         self._local_log_interp=log_interp
@@ -407,7 +435,7 @@ class Ktable5d(Data_table):
                 bounds_error=True)
 
     def set_kdata(self, new_kdata):
-        """Changes kdata. this is preferred to directly accessing kdata because one
+        """Changes kdata (inplace). this is preferred to directly accessing kdata because one
         could forget to run setup_interpolation().
 
         Parameters
@@ -440,6 +468,10 @@ class Ktable5d(Data_table):
                 controlled by self._settings._log_interp but only when the ktable is loaded.
                 If you change that after the loading, you should rerun setup_interpolation().
 
+        Returns
+        -------
+            array of shape (logp_array.size, self.Nw , self.Ng)
+                The interpolated kdata.
         """
         coord_to_interp=np.array([logp_array,t_array,np.log(x_array)]).transpose()
         tmp_res=self._finterp_kdata(coord_to_interp)
@@ -456,7 +488,8 @@ class Ktable5d(Data_table):
 
 
     def remap_logPT(self, logp_array=None, t_array=None, x_array= None):
-        """remap_logPT re-interpolates the kdata on a new temprature and log pressure grid. 
+        """remap_logPT re-interpolates the kdata on a new temprature and log pressure grid
+        (inplace). 
 
         Parameters
         ----------
@@ -560,7 +593,6 @@ class Ktable5d(Data_table):
             ax.plot(self.ggrid,toplot,**kwarg)
             ax.set_xlabel('g')
         ax.set_ylabel('Cross section ('+self.kdata_unit+')')
-        ax.grid(True)
         if yscale is not None: ax.set_yscale(yscale)
 
     def __repr__(self):
@@ -637,7 +669,7 @@ class Ktable5d(Data_table):
 
     def bin_down(self, wnedges=None, weights=None, ggrid=None,
         remove_zeros=False, num=300, use_rebin=False, write=0):
-        """Method to bin down a kcoeff table to a new grid of wavenumbers
+        """Method to bin down a kcoeff table to a new grid of wavenumbers (inplace).
 
         Parameters
         ----------
@@ -685,7 +717,7 @@ class Ktable5d(Data_table):
         self.setup_interpolation()
     
     def clip_spectral_range(self, wn_range=None, wl_range=None):
-        """Limits the data to the provided spectral range:
+        """Limits the data to the provided spectral range (inplace):
 
            * Wavenumber in cm^-1 if using wn_range argument
            * Wavelength in micron if using wl_range
@@ -694,6 +726,27 @@ class Ktable5d(Data_table):
         if self.kdata is not None:
             self.setup_interpolation()
 
+    def remap_g(self, ggrid=None, weights=None):
+        """Method to resample a kcoeff table to a new g grid (inplace).
+
+        Parameters
+        ----------
+            ggrid: array
+                New grid of abcissas for quadrature
+            weights: array
+                New grid of weights
+        """
+        weights=np.array(weights)
+        ggrid=np.array(ggrid)
+        self.Ng=weights.size
+        newkdata=np.zeros((self.Np, self.Nt, self.Nx, self.Nw, self.Ng))
+        g_sample_5d(ggrid, newkdata, self.ggrid, self.kdata)
+        self.ggrid=ggrid
+        self.weights=weights
+        self.gedges=np.concatenate(([0],np.cumsum(self.weights)))
+        self.set_kdata(newkdata)
+
+
 
 def read_Qdat(filename):
     """Reads Q.dat files LMDZ style and extract the vmr grid.
