@@ -9,7 +9,7 @@ import h5py
 import numpy as np
 import astropy.units as u
 from .data_table import Data_table
-from .util.interp import rebin_ind_weights
+from .util.interp import rebin_ind_weights, rm_molec
 from .util.cst import KBOLTZ
 from .hires_spectrum import Hires_spectrum
 from .util.filenames import create_fname_grid_Kspectrum_LMDZ, select_kwargs
@@ -49,10 +49,10 @@ class Xtable(Data_table):
             self.filename=filename
         elif filename_filters or mol is not None:
             self.filename=self._settings.list_files(*filename_filters, molecule = mol,
-                                 only_one=True, search_path=search_path)[0]
+                                 only_one=True, search_path=search_path, path_type='xtable')[0]
         if self.filename is not None:
             if self.filename.lower().endswith('pickle'):
-                self.read_pickle(filename=self.filename, mol=mol)
+                self.read_pickle(filename=self.filename)
             elif self.filename.lower().endswith(('.hdf5', '.h5')):
                 self.read_hdf5(filename=self.filename, mol=mol)
             elif self.filename.lower().endswith(('.dat')):
@@ -65,81 +65,6 @@ class Xtable(Data_table):
             kdata_unit=kdata_unit, file_kdata_unit=file_kdata_unit,
             remove_zeros=remove_zeros)
 
-    @property
-    def shape(self):
-        """Returns the shape of self.kdata
-        """
-        return np.array([self.Np,self.Nt,self.Nw])
-
-    def read_pickle(self, filename=None, mol=None):
-        """Initializes xsec table and supporting data from an Exomol pickle file
-
-        Parameters
-        ----------
-            filename : str
-                Relative or absolute name of the input pickle file
-            mol : str, optional
-                Overrides the name of the molecule to be put in the :class:`Xtable` object.
-        """
-        if filename is None: raise TypeError("You should provide an input pickle filename")
-        pickle_file=open(filename,'rb')
-        raw=pickle.load(pickle_file, encoding='latin1')
-        pickle_file.close()
-        
-        if mol is not None: 
-            self.mol=mol
-        else:
-            self.mol=raw['name']
-            if self.mol=='H2OP': self.mol='H2O'
-
-        self.tgrid=raw['t']
-        self.wns=raw['wno']
-        self.wnedges=np.concatenate( \
-            ([self.wns[0]],(self.wns[:-1]+self.wns[1:])*0.5,[self.wns[-1]]))
-        self.logk=False
-
-        #deals with the p grid and units
-        if 'p_unit' in raw.keys():
-            self.p_unit=raw['p_unit']
-        else:
-            self.p_unit='bar'
-        self.pgrid=raw['p']
-        self.logpgrid=np.log10(self.pgrid)
-
-        #deals with the k grid and units
-        if 'kdata_unit' in raw.keys():
-            self.kdata_unit=raw['kdata_unit']
-        else:
-            self.kdata_unit='cm^2/molec'
-        self.kdata=raw['xsecarr']
-
-        if 'wn_unit' in raw.keys(): self.wn_unit=raw['wn_unit']
-
-        self.Np,self.Nt,self.Nw=self.kdata.shape
-
-    def write_pickle(self, filename):
-        """Saves data in a pickle format
-
-        Parameters
-        ----------
-            filename: str
-                Relative or absolute name of the file to be created and saved
-        """
-        fullfilename=filename
-        if not filename.lower().endswith('.pickle'): fullfilename=filename+'.pickle'
-        pickle_file=open(fullfilename,'wb')
-        dictout={'name':self.mol,
-                 'p':self.pgrid,
-                 'p_unit':self.p_unit,
-                 't':self.tgrid,
-                 'wno':self.wns,
-                 'wn_unit':self.wn_unit,
-                 'xsecarr':self.kdata,
-                 'kdata_unit':self.kdata_unit}
-        #print(dictout)
-        pickle.dump(dictout,pickle_file,protocol=-1)
-        pickle_file.close()
-
     def read_hdf5(self, filename=None, mol=None):
         """Initializes k coeff table and supporting data from an Exomol hdf5 file
 
@@ -152,58 +77,75 @@ class Xtable(Data_table):
         """
         if (filename is None or not filename.lower().endswith(('.hdf5', '.h5'))):
             raise RuntimeError("You should provide an input hdf5 file")
-        f = h5py.File(filename, 'r')
-        if 'mol_name' in f:
-            self.mol=f['mol_name'][()]
-        elif 'mol_name' in f.attrs:
-            self.mol=f.attrs['mol_name']
-        else:
-            self.mol=os.path.basename(filename).split(self._settings._delimiter)[0]
-        if isinstance(self.mol, np.ndarray): self.mol=self.mol[0]
-        if mol is not None:
-            self.mol=mol
-        if 'bin_edges' in f:
-            self.wns=f['bin_edges'][...]
-            if 'units' in f['bin_edges'].attrs:
-                self.wn_unit=f['bin_edges'].attrs['units']
-        else:
-            self.wns=f['bin_centers'][...]
-            if 'units' in f['bin_centers'].attrs:
-                self.wn_unit=f['bin_centers'].attrs['units']
-        self.wnedges=np.concatenate(  \
-            ([self.wns[0]],(self.wns[:-1]+self.wns[1:])*0.5,[self.wns[-1]]))
-        self.kdata=f['xsecarr'][...]
-        self.kdata_unit=f['xsecarr'].attrs['units']
-        self.tgrid=f['t'][...]
-        self.pgrid=f['p'][...]
-        self.logpgrid=np.log10(self.pgrid)
-        self.p_unit=f['p'].attrs['units']
-        self.logk=False
-        f.close()  
+        with h5py.File(filename, 'r') as f:
+            if 'mol_name' in f:
+                self.mol=f['mol_name'][()]
+            elif 'mol_name' in f.attrs:
+                self.mol=f.attrs['mol_name']
+            else:
+                if mol is not None:
+                    self.mol=mol
+                else:
+                    self.mol=os.path.basename(filename).split(self._settings._delimiter)[0]
+            if isinstance(self.mol, np.ndarray): self.mol=self.mol[0]
+            if 'DOI' in f:
+                self.DOI=f['DOI'][()][0]
+            if 'bin_edges' in f:
+                self.wns=f['bin_edges'][...]
+                if 'units' in f['bin_edges'].attrs:
+                    self.wn_unit=f['bin_edges'].attrs['units']
+            else:
+                self.wns=f['bin_centers'][...]
+                if 'units' in f['bin_centers'].attrs:
+                    self.wn_unit=f['bin_centers'].attrs['units']
+            self.wnedges=np.concatenate(  \
+                ([self.wns[0]],(self.wns[:-1]+self.wns[1:])*0.5,[self.wns[-1]]))
+            self.kdata=f['xsecarr'][...]
+            self.kdata_unit=f['xsecarr'].attrs['units']
+            self.tgrid=f['t'][...]
+            self.pgrid=f['p'][...]
+            self.logpgrid=np.log10(self.pgrid)
+            self.p_unit=f['p'].attrs['units']
+            self.logk=False
         self.Np,self.Nt,self.Nw=self.kdata.shape
 
-    def write_hdf5(self, filename):
+    def write_hdf5(self, filename, compression="gzip", compression_level=9,
+        kdata_unit=None, p_unit=None, exomol_units=False):
         """Saves data in a hdf5 format
 
         Parameters
         ----------
             filename: str
                 Name of the file to be created and saved
+            exomol_units: bool (optional)
+                If True, data are converted back to
+                cm^2 and bar units before being written.
         """
         fullfilename=filename
         if not filename.lower().endswith(('.hdf5', '.h5')):
-            fullfilename=filename+'.hdf5'
-        compression="gzip"
-        f = h5py.File(fullfilename, 'w')
-        f.create_dataset("mol_name", data=self.mol)
-        f.create_dataset("p", data=self.pgrid,compression=compression)
-        f["p"].attrs["units"] = self.p_unit
-        f.create_dataset("t", data=self.tgrid,compression=compression)
-        f.create_dataset("xsecarr", data=self.kdata,compression=compression)
-        f["xsecarr"].attrs["units"] = self.kdata_unit
-        f.create_dataset("bin_edges", data=self.wns,compression=compression)
-        f["bin_edges"].attrs["units"] = self.wn_unit
-        f.close()
+            fullfilename=filename+'.h5'
+        with h5py.File(fullfilename, 'w') as f:
+            f.create_dataset("key_iso_ll", (1,), data=self.isotopolog_id)
+            f.create_dataset("mol_mass", (1,), data=self.molar_mass*1000.)
+            f["mol_mass"].attrs["units"] = 'AMU'
+            if exomol_units:
+                kdata_unit='cm^2/molecule'
+                p_unit='bar'
+            if kdata_unit is not None:
+                conv_factor=u.Unit(rm_molec(self.kdata_unit)).to(u.Unit(rm_molec(kdata_unit)))
+                data_to_write=self.kdata*conv_factor
+                f.create_dataset("xsecarr", data=data_to_write, compression=compression,
+                    compression_opts=compression_level)
+                f["xsecarr"].attrs["units"] = kdata_unit
+            else:
+                f.create_dataset("xsecarr", data=self.kdata, compression=compression,
+                    compression_opts=compression_level)
+                f["xsecarr"].attrs["units"] = self.kdata_unit
+
+            # where most of the data is actually written
+            self.write_hdf5_common(f, compression=compression, compression_level=compression_level,
+            p_unit=p_unit)
+            f["bin_edges"].attrs["long_name"] = 'Wavenumber grid'
 
     def read_exo_transmit(self, filename, mol=None):
         """Creates an xsec object from an exo_transmit like spectra.
@@ -257,7 +199,7 @@ class Xtable(Data_table):
         grid_p_unit='Pa', p_unit='unspecified',
         kdata_unit='unspecified', file_kdata_unit='unspecified',
         **kwargs):
-        """Loads an `Xtable` from high-resolution spectra.
+        """Loads an `Xtable` from high-resolution spectra (inplace).
 
         .. warning::
             By default, log pressures are specified in Pa in logpgrid!!! If you want
@@ -300,7 +242,7 @@ class Xtable(Data_table):
 
             spec_hr=Hires_spectrum(fname, file_kdata_unit=file_kdata_unit,
                 **select_kwargs(kwargs,['skiprows','wn_column','mult_factor',
-                    'kdata_column','data_type']))
+                    'kdata_column','data_type','binary','mass_amu']))
             # for later conversion, the real kdata_unit is in spec_hr.kdata_unit
             self.kdata_unit=spec_hr.kdata_unit
             was_xsec=(spec_hr.data_type=='xsec')
@@ -310,6 +252,7 @@ class Xtable(Data_table):
                 k_hr=k_hr*KBOLTZ*self.tgrid[iT]/self.pgrid[iP]
             if first:
                 self.wns=wn_hr[1:-1]  #to be consistent with kcorr
+                self.wnedges=0.5*(wn_hr[:-1]+wn_hr[1:])
                 self.Nw=self.wns.size
                 self.kdata=np.zeros((self.Np,self.Nt,self.Nw))
                 self.kdata[iP,iT]=k_hr[1:-1]
@@ -332,7 +275,7 @@ class Xtable(Data_table):
         self.convert_p_unit(p_unit=p_unit)
 
     def bin_down(self, wnedges=None, remove_zeros=False, write=0):
-        """Method to bin down a xsec table to a new grid of wavenumbers (in place)
+        """Method to bin down a xsec table to a new grid of wavenumbers (inplace).
 
         Parameters
         ----------
@@ -369,7 +312,7 @@ class Xtable(Data_table):
         if remove_zeros : self.remove_zeros(deltalog_min_value=10.)
 
     def sample(self, wngrid, remove_zeros=False, log_interp=None):
-        """Method to re sample a xsec table to a new grid of wavenumbers (in place)
+        """Method to re sample a xsec table to a new grid of wavenumbers (inplace).
 
         Parameters
         ----------
@@ -450,6 +393,12 @@ class Xtable(Data_table):
         res.copy_attr(self, cp_kdata=cp_kdata)
         return res
 
+    @property
+    def shape(self):
+        """Returns the shape of self.kdata
+        """
+        return np.array([self.Np,self.Nt,self.Nw])
+
     def __repr__(self):
         """Method to output header
         """
@@ -459,3 +408,69 @@ class Xtable(Data_table):
         shape        : {shape}
         """.format(shape=self.shape)
         return output
+
+    def read_pickle(self, filename=None):
+        """Initializes xsec table and supporting data from an Exomol pickle file
+
+        Parameters
+        ----------
+            filename : str
+                Relative or absolute name of the input pickle file
+            mol : str, optional
+                Overrides the name of the molecule to be put in the :class:`Xtable` object.
+        """
+        if filename is None: raise TypeError("You should provide an input pickle filename")
+        pickle_file=open(filename,'rb')
+        raw=pickle.load(pickle_file, encoding='latin1')
+        pickle_file.close()
+        
+        self.mol=raw['name']
+        if self.mol=='H2OP': self.mol='H2O'
+
+        self.tgrid=raw['t']
+        self.wns=raw['wno']
+        self.wnedges=np.concatenate( \
+            ([self.wns[0]],(self.wns[:-1]+self.wns[1:])*0.5,[self.wns[-1]]))
+        self.logk=False
+
+        #deals with the p grid and units
+        if 'p_unit' in raw.keys():
+            self.p_unit=raw['p_unit']
+        else:
+            self.p_unit='bar'
+        self.pgrid=raw['p']
+        self.logpgrid=np.log10(self.pgrid)
+
+        #deals with the k grid and units
+        if 'kdata_unit' in raw.keys():
+            self.kdata_unit=raw['kdata_unit']
+        else:
+            self.kdata_unit='cm^2/molec'
+        self.kdata=raw['xsecarr']
+
+        if 'wn_unit' in raw.keys(): self.wn_unit=raw['wn_unit']
+
+        self.Np,self.Nt,self.Nw=self.kdata.shape
+
+    def write_pickle(self, filename):
+        """Saves data in a pickle format
+
+        Parameters
+        ----------
+            filename: str
+                Relative or absolute name of the file to be created and saved
+        """
+        fullfilename=filename
+        if not filename.lower().endswith('.pickle'): fullfilename=filename+'.pickle'
+        pickle_file=open(fullfilename,'wb')
+        dictout={'name':self.mol,
+                 'p':self.pgrid,
+                 'p_unit':self.p_unit,
+                 't':self.tgrid,
+                 'wno':self.wns,
+                 'wn_unit':self.wn_unit,
+                 'xsecarr':self.kdata,
+                 'kdata_unit':self.kdata_unit}
+        #print(dictout)
+        pickle.dump(dictout,pickle_file,protocol=-1)
+        pickle_file.close()
